<?php
/**
 * Contains the Session_Manager class.
 *
 * @package Growmatik
 */

namespace Growmatik;

use Growmatik\Controller\User_Order_Controller;

/**
 * Class Session_Manager
 *
 * Retrieves analytics session details from cookies.
 *
 * @package Growmatik
 */
class Session_Manager {
	/**
	 * The parsed contents of the cookie
	 *
	 * @var object
	 */
	private $cookie;

	/**
	 * If override is called, the FE won't pick up and update $this->cookie->overrideUser until the next page refresh.
	 * Therefore for the duration of current page, we need to override it manually. This is only useful when one of
	 * `get_effective_*` functions of this class are explicitly called.
	 *
	 * Normal access to WP_User object properties won't include these, e.g first name, etc.
	 *
	 * @var object
	 */
	private static $override = null;

	/**
	 * Session_Manager constructor
	 */
	public function __construct() {
		$this->get_analytics_cookie();

		if ( self::$override ) {
			$this->cookie->overrideUser = self::$override;
		}
	}

	/**
	 * Get the permanent client id of the visitor
	 *
	 * @return string|null
	 */
	public function get_client_id() {
		return isset( $this->cookie->clientId ) ? $this->cookie->clientId : null;
	}

	/**
	 * Get the temporary session id of the visitor
	 *
	 * @return string|null
	 */
	public function get_session_id() {
		return isset( $this->cookie->sessionId ) ? $this->cookie->sessionId : null;
	}

	/**
	 * Put a reset session id request at the footer which will be picked up by the script.
	 *
	 * Does NOT change the session ID for the duration of the current request. But if FE is going to use it again,
	 * it should reset, like {@see $override}.
	 *
	 * @param $session_id
	 * @param string $reason
	 */
	public function reset_session_id( $session_id, $reason ) {
		$session_reset = array(
			'reason'    => $reason,
			'sessionId' => $session_id,
		);

		$echo_reset = function () use ( $session_reset ) {
			wp_localize_script( 'growmatik-if-then', '_growmatikSessionReset', $session_reset );
		};

		add_action( 'wp_enqueue_scripts', $echo_reset, 8 );
	}

	/**
	 * Override the user properties of the visitor; e.g upgrade them to a Guest Customer
	 *
	 * @param string $type
	 * @param string $username
	 * @param string $email
	 * @param string $first_name
	 * @param string $last_name
	 * @param string $registered
	 * @param WC_Order $order
	 */
	public function override_user_props( $type, $email, $username, $first_name, $last_name, $registered, $order ) {
		// Not overriding `id` because the user does not exist on WordPress in Guest Checkouts.

		// Set the current guest-customer order as the last order.
		$last_order = array(
			'paymentMethod' => $order->get_payment_method(),
			'currency'      => $order->get_currency(),
			'created'       => $order->get_date_created(),
			'totalAmount'   => $order->get_total(),
			'itemCount'     => $order->get_item_count(),
			'items'         => array(),
			'categories'    => array(),
		);

		foreach ( $order->get_items() as $item ) {
			$product_id                         = $item->get_product_id();
			$last_order['items'][ $product_id ] = $item->get_name();
			$categories                         = get_the_terms( $product_id, 'product_cat' );
			$last_order['categories']           = $last_order['categories'] + wp_list_pluck( $categories, 'name', 'term_id' );
		}

		$user_override = (object) array(
			'type'       => $type,
			'email'      => $username,
			'username'   => $username,
			'firstName'  => $first_name,
			'lastName'   => $last_name,
			'registered' => $registered,
			'noOfOrders' => 1,
			'lastOrder'  => $last_order,
		);

		self::$override             = $user_override;
		$this->cookie->overrideUser = $user_override;

		$echo_override = function () use ( $user_override ) {
			wp_localize_script( 'growmatik-if-then', '_growmatikUserOverride', (array) $user_override );
		};

		add_action( 'wp_enqueue_scripts', $echo_override, 8 );
	}

	/**
	 * Override the user properties of the visitor; e.g upgrade them to a Guest Customer
	 */
	public function fixup_user_ecommerce_details() {
		$override_purchased_product_ids = function () {
			$orders = new User_Order_Controller();
			return $orders->list_of_purchased_product_ids();
		};
		$override_total_purchased_items = function () {
			$orders = new User_Order_Controller();
			return $orders->get_total_purchased_items();
		};

		add_filter( 'growmatik_user_purchased_product_ids', $override_purchased_product_ids );
		add_filter( 'growmatik_user_total_purchased_items', $override_total_purchased_items );
	}

	/**
	 * Gets the effective overridden user property of Guest Subscribers or Guest Customers.
	 *
	 * @param string $property
	 *
	 * @return mixed|null
	 */
	public function get_effective_user_property( $property ) {
		if ( isset( $this->cookie->overrideUser->{$property} ) ) {
			return $this->cookie->overrideUser->{$property};
		}

		return null;
	}

	/**
	 * Gets the effective user if if set by the Front-end.
	 *
	 * @return int|null
	 */
	public function get_effective_user_id() {
		if ( isset( $this->cookie->overrideUser->id ) ) {
			return $this->cookie->overrideUser->id;
		}

		return null;
	}

	/**
	 * Gets the effective user type if set by the Front-end.
	 *
	 * @return string|null
	 */
	public function get_effective_user_type() {
		if ( isset( $this->cookie->overrideUser->type ) ) {
			return $this->cookie->overrideUser->type;
		}

		return null;
	}

	/**
	 * Parses the analytics cookie
	 *
	 * @return void
	 */
	private function get_analytics_cookie() {
		$this->cookie = json_decode( filter_input( INPUT_COOKIE, 'outstand' ) );
	}
}
