<?php

namespace Growmatik\Controller;

/**
 * Class Status_Controller
 *
 * Class to get essential site information, such as plugin status, versions, and other information necessary for site
 * syncs and health checks.
 *
 * Important: This class must not execute heavy logic, as it is called from the backend on multiple occasions.
 *            Any non-essential information (e.g suggestion requirements) must be collected by Extra_Data_Controller.
 *
 * Important: Collecting any additional data may be subject to regulations in the Privacy Policy and Terms of Use that
 *            users accept upon registration.
 *
 * @package Growmatik\Controller
 */
class Status_Controller {
	public function status( $request ) {

		if ( $request->get_header( 'X-Site-ID' ) != get_option( 'growmatik_site_id' ) ) { // phpcs:ignore
			return new \WP_Error( 'site_id_not_valid', esc_html__( 'Invalid Site ID', 'growmatik' ), array( 'status' => 403 ) );
		}

		global $wp_version;
		$woo = $this->woocommerce_information();

		$info = array(
			'active'          => true,
			'woocommerce'     => \Growmatik\is_woocommerce_active(),

			'paused'          => \Growmatik\Admin\Integration::is_paused(),

			'site_locale'     => get_locale(),

			'currency'        => isset( $woo['currency'] ) ? $woo['currency'] : null,
			'currency_symbol' => isset( $woo['currency_symbol'] ) ? $woo['currency_symbol'] : null,

			'meta'            => array(
				'wp_admin_url'        => admin_url(),
				'wp_version'          => $wp_version,
				'plugin_version'      => GROWMATIK_VERSION,
				'php_version'         => phpversion(),
				'wc_version'          => isset( $woo['wc_version'] ) ? $woo['wc_version'] : null,

				'permalink_structure' => get_option( 'permalink_structure' ),
				'terms_modified_at'   => \Growmatik\Modification_Handler::get_terms_last_modification(),

				'wp_is_multisite'     => (bool) is_multisite(),
			),
		);

		return $info;
	}

	private function woocommerce_information() {
		try {
			if ( \Growmatik\is_woocommerce_active() ) {
				global $woocommerce;
				return array(
					'wc_version'      => $woocommerce->version,
					'currency'        => get_woocommerce_currency(),
					'currency_symbol' => get_woocommerce_currency_symbol(),
				);
			}
		} catch ( \Throwable $throwable ) { // phpcs:ignore Generic.CodeAnalysis.EmptyStatement.DetectedCatch
			// Not handling the exception; do not include the info.
		}

		return array();
	}
}
