<?php
/**
 * Handles WooCommerce checkout action for both guest and logged-in users
 *
 * @package Growmatik
 */
// check repeated thank you after refresh.

namespace Growmatik\Controller;

use Growmatik;
use Growmatik\Event_Transport;
use Growmatik\Modification_Handler;
use Growmatik\Session_Manager;

/**
 * Class Woocommerce_Checkout_Controller
 *
 * Handles WooCommerce checkout action. Records session information and sends a checkout event to backend.
 *
 * @package Growmatik
 */
class Woocommerce_Checkout_Controller {
	/**
	 * The key for order metadata specifying whether the Checkout was handled or not to prevent the action being handled
	 * and network requests multiple times. It is necessary as `woocommerce_thankyou` action may run more than once: e.g
	 * twice on initial checkout because of our old code, and then again user is able to refresh the thankyou page.
	 *
	 * @link https://github.com/woocommerce/woocommerce/issues/7787
	 */
	const CHECKOUT_HANDLED_META = '_growmatik_checkout_handled';

	/**
	 * @var \WC_Order $order
	 */
	private $order;

	/**
	 * @var Session_Manager $session
	 */
	private $session;

	/**
	 * @var \WP_User|false $user
	 */
	private $user;

	/**
	 * Handle_Wooommerce_Checkout constructor.
	 */
	public function __construct() {
		if ( Growmatik\is_woocommerce_active() ) {
			add_action( 'template_redirect', array( $this, 'safe_handle_checkout' ), 10 );
		}
	}

	/**
	 * Wrapper around handle_checkout that tries to shield WP from its exceptions
	 */
	public function safe_handle_checkout() {
		if ( ! is_checkout() || empty( is_wc_endpoint_url( 'order-received' ) ) ) {
			return;
		}

		global $wp;

		$order_id = $wp->query_vars['order-received'];
		// Don't proceed when order ID not found.
		if ( empty( $order_id ) ) {
			return;
		}

		try {
			$this->handle_checkout( $order_id );
		} catch ( \Throwable $ex ) { //phpcs:ignore Generic.CodeAnalysis.EmptyStatement.DetectedCatch
			// Record $ex using Sentry
		}
	}

	/**
	 * Checks if checkout is not already handled then handles it
	 *
	 * @param mixed $order_id Order ID from Woocommerce.
	 */
	public function handle_checkout( $order_id ) {
		$this->order   = new \WC_Order( $order_id );
		$this->session = new Session_Manager();

		$this->find_existing_user()
			->record_order_meta_data()
			->reset_session_id()
			->override_user_type()
			->invalidate_ecommerce_cache()
			->fixup_ecommerce_data_passing();

		if ( ! $this->is_checkout_handled() ) {
			Modification_Handler::handle_checkout( $this->order );
			$this->send_checkout_event()
				->mark_checkout_handled();
		}
	}

	/**
	 * Finds and stores the existing user of the person that did a checkout.
	 *
	 * If the user is logged in do not use the billing email, doing so results in the logged-in user being considered a
	 * guest if they enter a different email address than their WordPress email on WooCommerce Checkout page.
	 *
	 * @return $this
	 */
	private function find_existing_user() {
		if ( 'guest' === \Growmatik\get_user_type() ) {
			$order_email = $this->order->get_billing_email();
			$this->user  = get_user_by( 'email', $order_email );

			// WooCommerce won't make Aliens a user of current site if they do a guest checkout; so neither do we.
			if (
				false !== $this->user &&
				is_multisite() &&
				! is_user_member_of_blog( $this->user->ID, get_current_blog_id() )
			) {
				$this->user = false;
			}
		} else {
			$this->user = wp_get_current_user();
		}

		return $this;
	}

	/**
	 * Record session and client information for the order
	 *
	 * @return $this
	 */
	public function record_order_meta_data() {
		$meta_prefix = \Growmatik\meta_site_prefix();

		if ( $this->order->get_meta( "{$meta_prefix}_session_id" ) ) {
			return $this;
		}

		$this->order->add_meta_data( "{$meta_prefix}_session_id", $this->session->get_session_id(), true );
		$this->order->add_meta_data( "{$meta_prefix}_client_id", $this->session->get_client_id(), true );
		$this->order->save();

		return $this;
	}

	/**
	 * Resets the session ID after purchase. The session ID will not change in this request.
	 *
	 * @return $this
	 */
	private function reset_session_id() {
		// We just want to reset the exact session the order was made in; So we must not use `$session->get_session_id`
		// because that will reset the session each time the user visits the thankyou page.
		$meta_prefix = \Growmatik\meta_site_prefix();
		$session_id  = $this->order->get_meta( "{$meta_prefix}_session_id" );

		$this->session->reset_session_id( $session_id, 'checkout' );

		return $this;
	}

	/**
	 * Determine if this is a Guest Checkout.
	 *
	 * Use this function because our definition of 'guest' is different from a WordPress logged in user WRT users that
	 * are a member of another blog on multi-site, and an existing member that logs out and buys things with the same
	 * email.
	 *
	 * @return bool
	 */
	private function is_guest_checkout() {
		return ( false === $this->user );
	}

	/**
	 * Overrides the user type. This override will be picked up by FE.
	 *
	 * @return $this
	 */
	private function override_user_type() {
		// In this specific function do not use is_guest_checkout due to 593/594: When people* are logged out we'd like
		// to temporarily upgrade their current session anyway. (*People above includes own leads/customers and aliens.)
		if ( 'guest' !== \Growmatik\get_user_type() ) {
			return $this;
		}

		$this->session->override_user_props(
			'customer',
			$this->order->get_billing_email(),
			$this->order->get_billing_email(),
			$this->order->get_billing_first_name(),
			$this->order->get_billing_last_name(),
			$this->order->get_date_created()->format( 'Y-m-d H:i:s' ),
			$this->order
		);

		return $this;
	}

	/**
	 * Invalidates the e-commerce cache for the user or guest customer.
	 *
	 * @return $this
	 */
	private function invalidate_ecommerce_cache() {
		if ( $this->user ) {
			User_Order_Controller::invalidate_user_cache( $this->user->ID );
		}

		User_Order_Controller::invalidate_guest_cache( $this->order->get_billing_email() );

		return $this;
	}

	/**
	 * Fixes the user products list that is passed to the Front-end.
	 *
	 * Since this logic is executed after the footer script output (i.e window._growmatik), we need to fix the value of
	 * window._growmatik.user.products that is passed to the front-end to make sure any newly purchased products appear
	 * in that list; which is exactly what this function does.
	 *
	 * @return $this
	 */
	private function fixup_ecommerce_data_passing() {
		$this->session->fixup_user_ecommerce_details();

		return $this;
	}

	/**
	 * Reports a checkout event to backend.
	 *
	 * @return $this
	 */
	private function send_checkout_event() {
		// Select only main order data.
		$order_data = array_intersect_key( $this->order->get_data(), array_flip( $this->order->get_data_keys() ) );

		// Plus the id.
		$order_data['id'] = $this->order->get_id();

		// Prepare and send the event.
		$event_name = $this->is_guest_checkout() ? 'wpGuestCheckout' : 'wpUserCheckout';
		$event_data = array(
			'order' => $order_data,
		);

		// If user is not logged in but her account already exists, it means she is now a customer and WooCommerce will
		// -or- already made her a member of the current blog in multi-site.  However Event_Transport still thinks of
		// her as a Guest, so we force the user type.
		$event_user_id   = $this->is_guest_checkout() ? null : $this->user->ID;
		$event_user_type = $this->is_guest_checkout() ? null : 'customer';

		$transport = new Event_Transport( $event_user_id, $event_user_type );
		$transport->send_event( $event_name, $event_data );

		return $this;
	}


	/**
	 * Checks if the checkout is handled.
	 *
	 * @return bool
	 */
	private function is_checkout_handled() {
		return (bool) $this->order->get_meta( self::CHECKOUT_HANDLED_META );
	}

	/**
	 * Sets metadata on the order specifying the checkout was handled.
	 *
	 * @return $this
	 */
	private function mark_checkout_handled() {
		$this->order->add_meta_data( self::CHECKOUT_HANDLED_META, '1', true );
		$this->order->save();

		return $this;
	}
}
