<?php


namespace Growmatik;

use Growmatik\Controller\User_Order_Controller;

function get_user_type() {
	if ( ! is_user_logged_in() ) {
		return 'guest';
	}

	if ( is_multisite() ) {
		if ( ! is_user_member_of_blog( get_current_user_id(), get_current_blog_id() ) ) {
			return 'guest';
		}
	}

	if ( ! is_woocommerce_active() ) {
		return 'lead';
	}

	$orders = new User_Order_Controller();

	return $orders->get_user_order_count( get_current_user_id() ) > 0 ? 'customer' : 'lead';
}

function is_woocommerce_active() {
	return did_action( 'woocommerce_loaded' );
}

function starts_with( $haystack, $needle ) {
	return substr( $haystack, 0, strlen( $needle ) ) === $needle;
}

function error_log( $log ) {
	if ( is_array( $log ) || is_object( $log ) ) {
		\error_log( print_r( $log, true ) ); // phpcs:ignore
		return;
	}

	\error_log( $log ); // phpcs:ignore
}

/**
 * Checks whether $search is an array and all $keys exist in it, whether null or non-null.
 *
 * @param array $keys
 * @param array $search
 *
 * @return bool
 */
function array_keys_exist( $keys, $search ) {
	if ( ! is_array( $search ) ) {
		return false;
	}

	$count = 0;
	foreach ( $keys as $key ) {
		$count += ( isset( $search[ $key ] ) || array_key_exists( $key, $search ) ) ? 1 : 0;
	}

	return count( $keys ) === $count;
}

/**
 * Returns a string to be used as the meta key prefix for Growmatik meta entries in places where the meta must only be
 * available for a specific site and must not be synced to other sites. As an example if tenant re-integrates with a new
 * site id, we don't want to use the old site's purchase session ids with the new site. Therefore append site_id to
 * order metas and other metas which should not survive on changing sites.
 *
 * @return string
 */
function meta_site_prefix() {
	$site_id = get_option( 'growmatik_site_id' );
	return "_growmatik_{$site_id}";
}

function get_relative_permalink( $post ) {
	$permalink = get_permalink( $post );
	$url       = str_replace( home_url(), '', $permalink );

	return rtrim( $url, '/' );
}

function is_screen( $screen_id ) {
	$screen = get_current_screen();
	if ( ! $screen ) {
		return false;
	}

	return $screen_id === $screen->id;
}

/**
 * Utility function to verify nonces without causing warnings in WPCS.
 *
 * @param string $action
 *
 * @param string $field_name
 *
 * @return bool
 */
function verify_get_nonce( $action, $field_name = 'nonce' ) {
	if ( ! isset( $_GET[ $field_name ] ) ) {
		return false;
	}

	// phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
	if ( ! wp_verify_nonce( wp_unslash( $_GET[ $field_name ] ), $action ) ) {
		return false;
	}

	return true;
}

/**
 * Utility function to verify nonces without causing warnings in WPCS.
 *
 * @param string $action
 *
 * @param string $field_name
 *
 * @return bool
 */
function verify_post_nonce( $action, $field_name = 'nonce' ) {
	if ( ! isset( $_POST[ $field_name ] ) ) {
		return false;
	}

	// phpcs:ignore WordPress.Security.ValidatedSanitizedInput.InputNotSanitized
	if ( ! wp_verify_nonce( wp_unslash( $_POST[ $field_name ] ), $action ) ) {
		return false;
	}

	return true;
}

/**
 * Get WooCommerce user persistent cart items.
 *
 * @param string $user_id User ID.
 *
 * @return array Cart items.
 */
function get_user_cart_items( $user_id ) {
	$wc_meta_key = '_woocommerce_persistent_cart_' . get_current_blog_id();
	$user_meta   = get_user_meta( $user_id );
	$cart_items  = array();

	foreach ( $user_meta as $meta_key => $meta_value ) {
		// Check all meta keys starting with '_woocommerce_persistent_cart'.
		if ( starts_with( $meta_key, $wc_meta_key ) ) {
			// Found a persistent cart; check if empty.
			// phpcs:ignore WordPress.PHP.DiscouragedPHPFunctions.serialize_unserialize
			$cart = maybe_unserialize( $meta_value[0] );

			// If a cart exists, $cart['cart'] will be set and contain an item otherwise will be empty.
			// If found a cart, return it; otherwise continue checking the rest of the metas.
			if ( ! empty( $cart['cart'] ) ) {
				$cart_items = array_merge(
					$cart_items,
					array_column( $cart['cart'], 'product_id' )
				);
			}
		}
	}

	return $cart_items;
}

/**
 * Get current user cart items.
 *
 * Retrieve the current visitor cart items via WC session, supports
 * both guest and logged in user.
 *
 * @return array Cart items.
 */
function get_cart_items() {
	$cart_items = array();
	// Requires WC to be loaded first.
	if ( ! function_exists( 'wc' ) ) {
		return $cart_items;
	}

	$wc_cart_items = wc()->cart->get_cart();
	if ( ! empty( $wc_cart_items ) ) {
		$cart_items = array_column( $wc_cart_items, 'product_id' );
	}

	return $cart_items;
}
